package org.hardknockjava.gettysburgaddress;

import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.TreeMap;
import java.util.TreeSet;

public class GettysburgAddress {
    private static final String address = "Fourscore and seven years ago " //
            + "our fathers brought forth on this continent a new nation " //
            + "conceived in liberty and dedicated to the proposition " //
            + "that all men are created equal";

    // This map holds counts for each letter that appears.
    // We use a TreeMap so the letters will be accessed in
    // ascending order.
    private Map<Character, Integer> letterCountMap = new TreeMap<Character, Integer>();
    
    // This map holds Sets of the words in which each letter appears.
    // We use a TreeMap so the letters will be accessed in
    // ascending order.
    private Map<Character, Set<String>> wordsByLetterMap = new TreeMap<Character, Set<String>>();

    public static void main(String[] args) {
        GettysburgAddress gettysburgAddress = new GettysburgAddress();

        // Populate and display letterCountMap.
        gettysburgAddress.countLetters();
        
        System.out.println("\n\n");
        
        // Populate and display wordsByLetterMap.
        gettysburgAddress.groupWordsByLetter();
    }

    private void countLetters() {
        String shiftedAddress = address.toUpperCase();
        
        for (int i = 0; i < shiftedAddress.length(); i++) {
            Character key = shiftedAddress.charAt(i);
            
            // Find the value (Integer) associated with the letter.
            if (key != ' ') {
                Integer value = letterCountMap.get(key);
                
                // If no value is associated with the letter yet,
                // set value to 1. Otherwise increment it.
                if (value == null) {
                    value = 1;
                } else {
                    value = value.intValue() + 1;
                }
                
                // Add/replace the value associated with the letter.
                letterCountMap.put(key, value);
            }
        }

        // Display each letter and the associated count.
        for (Entry<Character, Integer> entry : letterCountMap.entrySet()) {
            System.out.println(entry.getKey() + "  " + entry.getValue());
        }
    }

    private void groupWordsByLetter() {
        String shiftedAddress = address.toUpperCase();
        
        // Divide the quote into individual words.
        String[] wordArray = shiftedAddress.split(" ");

        for (String word : wordArray) {
            
            // Process each letter in the word.
            for (int i = 0; i < word.length(); i++) {
                Character key = word.charAt(i);
                if (key != ' ') {
                    
                    // Get the value (Set) associated with the letter.
                    Set<String> value = wordsByLetterMap.get(key);
                    
                    // If the letter doesn't appear in the map yet,
                    // allocate a Set and store it in the map keyed
                    // by the letter. Use a TreeSet so the values
                    // will be displayed in ascending order.
                    if (value == null) {
                        value = new TreeSet<String>();
                        wordsByLetterMap.put(key, value);
                    }
                    
                    // Add the word to the Set.
                    value.add(word);
                }
            }
        }

        // Display the map.
        for (Entry<Character, Set<String>> entry : wordsByLetterMap.entrySet()) {
            System.out.println(entry.getKey());
            for (String word : entry.getValue()) {
                System.out.println("   " + word);
            }
        }
    }
}
